#!/usr/bin/env python

import math

# FYI there are 8 of them
_BLOCKS = "▏▎▍▌▋▊▉█"
_MAX_WIDTH = 128

def sparkbarh(numbers=[], width=8, add_num=False):
    assert width > 0, "Requires non-negative width"
    assert width <= _MAX_WIDTH, "Max width is 128 characters."

    if len(numbers) == 0:
        return []

    # find max value
    max_value = _find_max_value(numbers)
    if max_value is None:
        return [""] * len(numbers)

    has_negative_numbers = _has_negative_numbers(numbers)
    assert has_negative_numbers == False, "There are negative numbers"

    return [sparkbarh_cell(x, max_value, width, add_num) for x in iter(numbers)]


def _find_max_value(numbers):
    not_none = [x for x in numbers if x is not None]
    if len(not_none) > 0:
        return max(not_none)
    else:
        return None

def _has_negative_numbers(numbers):
    for number in numbers:
        if number is not None and number < 0:
            return True
    return False

def sparkbarh_cell(value, max_value, width=8, add_num=False):
    assert width > 0, "Requires non-negative width"
    assert width <= _MAX_WIDTH, "Max width is 128 characters."

    if value is None:
        return ""
    assert value <= max_value, "Value to draw exceeds max value"

    bar_length = width * (value / max_value)
    full_bars = _BLOCKS[7] * math.floor(bar_length)
    remain_bar_length = bar_length % 1

    remain_bar_index = round(remain_bar_length * 8) - 1
    if remain_bar_length > 0 and remain_bar_index >= 0:
        partial_bars = _BLOCKS[remain_bar_index]
    else:
        partial_bars = ""

    if add_num:
        return "{}{} {}".format(full_bars, partial_bars, value)
    else:
        return "{}{}".format(full_bars, partial_bars)




#!/usr/bin/env python

import pytest

from sparkbar import sparkbarh

def test_empty_list():
    "Test passing in an empty list"
    assert sparkbarh([]) == []

def test_list_with_negative_numbers():
    "Raises on negative numbers"
    with pytest.raises(Exception) as e_info:
        sparkbarh([1, -1, 1])

def test_list_of_none():
    "List full of nones returns blank strings"
    assert sparkbarh([None, None, None]) == ["", "", ""]

def test_mult_number_list():
    "Basic number list is correctly visualized"
    in_list = [40, 30, 10]
    assert sparkbarh(in_list, width=4) == ["████", "███", "█"]

def test_single_number_list():
    "Single number list is correctly visualized"
    in_list = [30]
    assert sparkbarh(in_list, width=5) == ["█████"]

def test_default_width():
    "Not passing a width works"
    in_list = [30, 15]
    assert sparkbarh(in_list) == ["████████", "████"]

def test_list_with_none():
    "List with None values is correctly visualized"
    in_list = [40, None, 10]
    assert sparkbarh(in_list, width=4) == ["████", "", "█"]

def test_add_num():
    "add_num option displays correctly"
    in_list = [40, 30, 10]
    assert sparkbarh(in_list, width=4, add_num=True) == ["████ 40", "███ 30", "█ 10"]

def test_negative_width():
    "Raises on negative widths"
    with pytest.raises(Exception) as e_info:
        sparkbarh([1, 2, 3], width=-1)

def test_excessive_width():
    "Raises on large widths"
    with pytest.raises(Exception) as e_info:
        sparkbarh([1, 2, 3], width=150)



#!/usr/bin/env python

import setuptools

setuptools.setup(
    name='sparkbar',
    version='1.0.1',
    description='Creates text version of horizontal bar chart.',
    author='Kyle Demeule',
    author_email='kyle.demeule@gmail.com',
    packages=setuptools.find_packages(exclude='test'),
    install_requires=[
        "pytest"
    ],
)